class Game.Objects.Ball extends Engine.PongObject
{
   var direction;
   var __defaultSpeed;
   var __slowSpeed;
   var __fastSpeed;
   var target;
   var lastPos;
   var currentPos;
   var __stuckToObj;
   var world;
   var motionVec;
   var motionDist;
   var motionDir;
   var motionLeft;
   var motionRight;
   var motionTop;
   var motionBottom;
   var guide;
   static var depthShift = 5000;
   var assetID = "Ball";
   var className = "Ball";
   var radius = 0;
   var __speed = Game.PongLevels.DEFAULT_SPEED;
   var __speedOffset = Game.PongLevels.BALL_SPEED_OFFSET;
   var __angle = Game.PongLevels.DEFAULT_ANGLE;
   var __missile_factor = Game.PongLevels.BALL_MISSILE_FACTOR;
   static var MIN_ANGLE = Game.PongLevels.MIN_BALL_ANGLE;
   var __stuck = false;
   var __blast = false;
   static var __inited = false;
   var __assetFrame = 1;
   var __active = true;
   var __blastChanged = false;
   function Ball(mySpeed, myAngle, myStuck)
   {
      super();
      this.direction = new Vector(200,0,0);
      if(myStuck != undefined)
      {
         this.__stuck = myStuck;
      }
      if(mySpeed != undefined)
      {
         this.__speed = mySpeed;
      }
      this.__defaultSpeed = this.__speed;
      this.__slowSpeed = this.__defaultSpeed - this.__speedOffset;
      this.__fastSpeed = this.__defaultSpeed + this.__speedOffset;
      this.angle = myAngle;
   }
   function set defaultSpeed(mySpeed)
   {
      this.__defaultSpeed = mySpeed;
      this.__slowSpeed = this.__defaultSpeed - this.__speedOffset;
      this.__fastSpeed = this.__defaultSpeed + this.__speedOffset;
      this.speed = this.__defaultSpeed;
   }
   function get defaultSpeed()
   {
      return this.__defaultSpeed;
   }
   function set active(flag)
   {
      this.__active = flag;
      this.target._visible = flag;
      if(flag)
      {
         this.getUpdates();
      }
      else
      {
         this.cancelUpdates();
      }
   }
   function get active()
   {
      return this.__active;
   }
   function set angle(newAngle)
   {
      this.__angle = newAngle;
      this.calcDirection();
   }
   function get angle()
   {
      return this.__angle;
   }
   function set speed(newSpeed)
   {
      this.__speed = newSpeed;
      this.calcDirection();
   }
   function get speed()
   {
      return this.__speed;
   }
   function set blast(newBlast)
   {
      this.__blast = newBlast;
      if(!this.__blast)
      {
         this.__assetFrame = 1;
         this.speed = this.__defaultSpeed;
      }
      else
      {
         this.__assetFrame = 3;
         this.speed = this.__fastSpeed * this.__missile_factor;
      }
      this.__blastChanged = true;
      this.queueForDisplay();
   }
   function get blast()
   {
      return this.__blast;
   }
   function set stuck(newStuck)
   {
      this.__stuck = newStuck;
      this.lastPos = this.currentPos;
      this.calcNewMotionProps();
      if(!this.blast)
      {
         this.calcDirection();
      }
      else
      {
         var _loc2_ = this.world.getMissileTarget(this.__stuckToObj.orientation);
         var _loc3_ = _loc2_.top + Math.random() * _loc2_.width;
         this.direction = new Vector(_loc2_.x - this.lastPos.x,_loc3_ - this.lastPos.y,0);
         this.direction.length = this.speed;
         this.__angle = this.vectorToAngle(this.direction);
      }
      if(!this.__stuck)
      {
         this.__stuckToObj = null;
      }
   }
   function get stuck()
   {
      return this.__stuck;
   }
   function stickTo(gameObj)
   {
      this.__stuckToObj = gameObj;
      if(this.__stuckToObj == null)
      {
         this.stuck = false;
         return undefined;
      }
      this.stuck = true;
      this.stickToObj();
   }
   function deflect(newPos, normal, deflectToLeft)
   {
      var _loc7_ = this.__angle;
      var _loc3_ = normal.dot(this.direction);
      this.direction = new Vector(this.direction.x - normal.x * _loc3_ * 2,this.direction.y - normal.y * _loc3_ * 2,0);
      var _loc2_ = this.vectorToAngle(this.direction);
      while(_loc2_ < 0)
      {
         _loc2_ += 360;
      }
      if(deflectToLeft == undefined)
      {
         if(_loc2_ < Game.Objects.Ball.MIN_ANGLE)
         {
            this.angle = Game.Objects.Ball.MIN_ANGLE;
         }
         else if(180 - Game.Objects.Ball.MIN_ANGLE < _loc2_ && _loc2_ < 180)
         {
            this.angle = 180 - Game.Objects.Ball.MIN_ANGLE;
         }
         else if(180 < _loc2_ && _loc2_ < 180 + Game.Objects.Ball.MIN_ANGLE)
         {
            this.angle = 180 + Game.Objects.Ball.MIN_ANGLE;
         }
         else if(360 - Game.Objects.Ball.MIN_ANGLE < _loc2_)
         {
            this.angle = 360 - Game.Objects.Ball.MIN_ANGLE;
         }
         else
         {
            this.__angle = _loc2_;
         }
      }
      else if(180 - Game.Objects.Ball.MIN_ANGLE < _loc2_ && _loc2_ < 180 + Game.Objects.Ball.MIN_ANGLE)
      {
         if(deflectToLeft)
         {
            this.angle = 180 + Game.Objects.Ball.MIN_ANGLE;
         }
         else
         {
            this.angle = 180 - Game.Objects.Ball.MIN_ANGLE;
         }
      }
      else if(Game.Objects.Ball.MIN_ANGLE > _loc2_ || _loc2_ > 360 - Game.Objects.Ball.MIN_ANGLE)
      {
         if(deflectToLeft)
         {
            this.angle = 360 - Game.Objects.Ball.MIN_ANGLE;
         }
         else
         {
            this.angle = Game.Objects.Ball.MIN_ANGLE;
         }
      }
      else
      {
         this.__angle = _loc2_;
      }
      this.calcNewPosTo(newPos.x,newPos.y,true);
   }
   function deflectH(newPos, normalAngle)
   {
      if(normalAngle == undefined)
      {
         if(this.direction.y > 0)
         {
            normalAngle = 0;
         }
         else
         {
            normalAngle = 180;
         }
      }
      var _loc3_ = this.angleToVector(normalAngle);
      this.deflect(newPos,_loc3_);
   }
   function deflectV(newPos, normalAngle)
   {
      var _loc3_ = this.direction.x > 0;
      if(normalAngle == undefined)
      {
         if(_loc3_)
         {
            normalAngle = 270;
         }
         else
         {
            normalAngle = 90;
         }
      }
      if(this.blast)
      {
         this.blast = false;
      }
      var _loc4_ = this.angleToVector(normalAngle);
      this.deflect(newPos,_loc4_,_loc3_);
   }
   function update(elapsed)
   {
      if(this.blast)
      {
         if(this.__stuck)
         {
            if(this.__stuckToObj.orientation == Engine.PongObject.RIGHT_DIR)
            {
               this.target._xscale = 100;
            }
            else
            {
               this.target._xscale = -100;
            }
         }
         else
         {
            this.target.gotoAndStop(3);
            if(this.direction.x >= 0)
            {
               this.target._xscale = -100;
            }
            else
            {
               this.target._xscale = 100;
            }
         }
      }
      else
      {
         this.target._xscale = 100;
      }
      if(this.__stuck)
      {
         this.stickToObj();
         return undefined;
      }
      this.calcNewPosBy(this.direction.x * elapsed,this.direction.y * elapsed);
      if(!this.checkBricks() && !this.checkPaddles())
      {
         this.checkBounds(elapsed);
      }
      this.moveBy(this.motionVec.x,this.motionVec.y,0);
      this.updateBorders();
   }
   function onAddToScene()
   {
      this.currentPos = new Vector(this.x,this.y,0);
      this.width = this.target._width - 2;
      this.height = this.target._height - 2;
      this.radius = this.height / 2;
      this.getUpdates();
   }
   function updateBorders()
   {
      this.left = this.x - this.radius;
      this.right = this.x + this.radius;
      this.top = this.y - this.radius;
      this.bottom = this.y + this.radius;
   }
   function onDisplay()
   {
      this.updateBorders();
      if(this.__blastChanged)
      {
         if(this.__stuck && this.blast)
         {
            this.target.gotoAndStop(2);
         }
         else
         {
            this.target.gotoAndStop(this.__assetFrame);
         }
         this.__blastChanged = false;
      }
   }
   function activateItem(item)
   {
      if(item == Game.PongLevels.BALL_SLOW)
      {
         this.speed = this.__slowSpeed;
      }
      else if(item == Game.PongLevels.BALL_FAST)
      {
         this.speed = this.__fastSpeed;
      }
   }
   function deactivateItem()
   {
      this.speed = this.__defaultSpeed;
   }
   function calcNewPosTo(newX, newY, keepLastPos)
   {
      if(!keepLastPos)
      {
         this.lastPos = this.currentPos;
      }
      this.currentPos = new Vector(newX,newY,0);
      this.calcNewMotionProps();
   }
   function calcNewPosBy(dx, dy, keepLastPos)
   {
      if(!keepLastPos)
      {
         this.lastPos = this.currentPos;
      }
      this.currentPos = new Vector(this.lastPos.x + dx,this.lastPos.y + dy,0);
      this.calcNewMotionProps();
   }
   function stickToObj()
   {
      var _loc2_ = undefined;
      if(this.__stuckToObj.orientation == Engine.PongObject.RIGHT_DIR)
      {
         _loc2_ = this.__stuckToObj.left - this.radius;
      }
      else
      {
         _loc2_ = this.__stuckToObj.right + this.radius;
      }
      this.moveTo(_loc2_,this.__stuckToObj.y,0);
      this.currentPos = new Vector(this.x,this.y,0);
   }
   function inReach(point)
   {
      var _loc2_ = new Vector(point.x - this.lastPos.x,point.y - this.lastPos.y,0);
      return _loc2_.length <= this.motionDist;
   }
   function checkBounds(elapsed)
   {
      this.world.checkBounds(this,elapsed);
   }
   function checkPaddles()
   {
      return this.world.checkPaddles(this);
   }
   function checkBricks()
   {
      return this.world.checkBricks(this);
   }
   function calcDirection()
   {
      this.direction = this.angleToVector(this.angle);
      this.direction.length = this.__speed;
   }
   function angleToVector(angle)
   {
      var _loc1_ = angle / 180 * 3.141592653589793;
      return new Vector(Math.sin(_loc1_),- Math.cos(_loc1_),0);
   }
   function vectorToAngle(v)
   {
      return Math.atan2(v.y,v.x) / 3.141592653589793 * 180 + 90;
   }
   function calcNewMotionProps()
   {
      this.motionVec = new Vector(this.currentPos.x - this.lastPos.x,this.currentPos.y - this.lastPos.y,0);
      this.motionDist = this.motionVec.length;
      this.motionDir = this.motionVec.getNormalized();
      this.motionLeft = Math.min(this.currentPos.x,this.lastPos.x) - this.radius;
      this.motionRight = Math.max(this.currentPos.x,this.lastPos.x) + this.radius;
      this.motionTop = Math.min(this.currentPos.y,this.lastPos.y) - this.radius;
      this.motionBottom = Math.max(this.currentPos.y,this.lastPos.y) + this.radius;
   }
   function showAngles(newPos, normal)
   {
      if(!this.guide)
      {
         this.guide = this.target.createEmptyMovieClip("guide",0);
      }
      this.guide.clear();
      if(newPos)
      {
         var _loc3_ = new Vector(this.lastPos.x - newPos.x,this.lastPos.y - newPos.y,0);
         _loc3_.length = 25;
         this.guide.lineStyle(0,65280);
         this.guide.moveTo(0,0);
         this.guide.lineTo(_loc3_.x,_loc3_.y);
      }
      if(normal)
      {
         var _loc2_ = normal.copy();
         _loc2_.length = 20;
         this.guide.lineStyle(0,16711680);
         this.guide.moveTo(0,0);
         this.guide.lineTo(_loc2_.x,_loc2_.y);
      }
      var _loc4_ = this.direction.copy();
      _loc4_.length = 30;
      this.guide.lineStyle(0,255);
      this.guide.moveTo(0,0);
      this.guide.lineTo(_loc4_.x,_loc4_.y);
   }
}
